from huscy.project_design.models import DataAcquisitionMethod, Experiment, Session


def add_data_acquisition_method(session, type, location, order=None):
    if order is None:
        order = DataAcquisitionMethod.objects.filter(session=session).count() + 1

    return DataAcquisitionMethod.objects.create(
        session=session,
        type=type,
        location=location,
        order=order
    )


def create_experiment(project, title='', description='', order=0):
    project_experiment_count = project.experiments.count()

    return Experiment.objects.create(
        description=description,
        order=order or project_experiment_count + 1,
        project=project,
        title=title or f'Experiment {project_experiment_count+1}',
    )


def create_session(experiment, duration, operator, title='', order=0):
    experiment_session_count = experiment.sessions.count()
    project_session_count = Session.objects.filter(experiment__project=experiment.project).count()

    return Session.objects.create(
        duration=duration,
        experiment=experiment,
        operator=operator,
        order=order or experiment_session_count + 1,
        title=title or f'Session {project_session_count + 1}'
    )


def get_data_acquisition_methods(project=None):
    queryset = DataAcquisitionMethod.objects
    if project:
        queryset = queryset.filter(session__experiment__project=project)
    return queryset.order_by('pk')


def get_experiments(project=None):
    queryset = Experiment.objects
    if project:
        queryset = queryset.filter(project=project)
    return queryset.all()


def get_sessions():
    return Session.objects.all()



from rest_framework_nested.routers import DefaultRouter

from huscy.project_design import views
from huscy.projects.urls import project_router


router = DefaultRouter()
router.register('dataacquisitionmethods', views.DataAcquisitionMethodViewSet)
router.register('sessions', views.SessionViewSet)

project_router.register('experiments', views.ExperimentViewSet, basename='experiment')

urlpatterns = []
urlpatterns += router.urls
urlpatterns += project_router.urls



from rest_framework import serializers

from huscy.project_design.models import DataAcquisitionMethod, Experiment, Session
from huscy.project_design.services import (
    add_data_acquisition_method,
    create_experiment,
    create_session,
)


class DataAcquisitionMethodSerializer(serializers.ModelSerializer):
    class Meta:
        model = DataAcquisitionMethod
        fields = (
            'id',
            'location',
            'order',
            'session',
            'type',
        )

    def create(self, validated_data):
        return add_data_acquisition_method(**validated_data)


class SessionSerializer(serializers.ModelSerializer):
    order = serializers.IntegerField(required=False)
    title = serializers.CharField(required=False)

    class Meta:
        model = Session
        fields = (
            'duration',
            'experiment',
            'id',
            'operator',
            'order',
            'title',
        )

    def create(self, validated_data):
        return create_session(**validated_data)

    def to_representation(self, session):
        response = super().to_representation(session)
        response['data_acquisition_methods'] = \
            DataAcquisitionMethodSerializer(session.dataacquisitionmethod_set.all(), many=True).data
        return response


class ExperimentSerializer(serializers.ModelSerializer):
    order = serializers.IntegerField(required=False)
    title = serializers.CharField(required=False)

    class Meta:
        model = Experiment
        fields = (
            'description',
            'id',
            'order',
            'project',
            'title',
        )
        read_only_fields = 'project',

    def create(self, validated_data):
        return create_experiment(**validated_data)

    def to_representation(self, experiment):
        response = super().to_representation(experiment)
        response['sessions'] = SessionSerializer(experiment.sessions.all(), many=True).data
        return response



from django.shortcuts import get_object_or_404
from rest_framework import mixins, viewsets
from rest_framework.permissions import IsAuthenticated

from huscy.project_design import serializer, services
from huscy.projects.models import Project


class DataAcquisitionMethodViewSet(mixins.CreateModelMixin, mixins.DestroyModelMixin,
                                   mixins.UpdateModelMixin, viewsets.GenericViewSet):
    queryset = services.get_data_acquisition_methods()
    serializer_class = serializer.DataAcquisitionMethodSerializer
    permission_classes = (IsAuthenticated, )


class ExperimentViewSet(mixins.CreateModelMixin, mixins.DestroyModelMixin, mixins.ListModelMixin,
                        mixins.UpdateModelMixin, viewsets.GenericViewSet):
    serializer_class = serializer.ExperimentSerializer
    permission_classes = (IsAuthenticated, )

    def initial(self, request, *args, **kwargs):
        self.project = get_object_or_404(Project, pk=self.kwargs['project_pk'])
        super().initial(request, *args, **kwargs)

    def get_queryset(self):
        return services.get_experiments(self.project)

    def perform_create(self, serializer):
        serializer.save(project=self.project)


class SessionViewSet(viewsets.ModelViewSet):
    queryset = services.get_sessions()
    serializer_class = serializer.SessionSerializer
    permission_classes = (IsAuthenticated, )
